// This application demonstrates sending a request message to a named queue
// and awaiting a response.  For the response to arrive, a partner application
// must receive the request and generate the response.  Program EXMQGET2 is
// used for this purpose.
//
// If no response arrives within 15 seconds, the request is assumed to have
// timed out.
//
//      program name:  exmqreq1 - example of MQPUT to write a message to a queue
//                                and receive a reply in a second queue         
//                
//                exmqreq1 has 3 parameters:
//                - the name of a request queue (required)
//                - the name of a response queue (required)
//                - the name of a queue manager (required)
//
//    Usage: exmqreq1 RequestQueueName ResponseQueueName QmgrName
//    
//    example command line:   exmqreq1 LEXQ1 LEXQ2 EXQMAN

using System;
using IBM.WMQ;

namespace exmqreq1
{
  /// <summary>
  /// Summary description for exmqreq1.
  /// </summary>
  class exmqreq1
  {
    /// <summary>
    /// The main entry point for the application.
    /// </summary>
    [STAThread]
    static void Main(String[] args)
    {
      MQQueueManager      mqQMgr;            // MQQueueManager instance
      MQQueue             requestQueue;      // MQQueue instance
      MQQueue             responseQueue;     // MQQueue instance
      MQRCText            mqrcText;          // MQRCText instance
      String              requestQueueName;  // Name of queue to use for requests
      String              responseQueueName; // Name of queue to use for requests

      if (args.Length == 0) 
      {
        System.Console.WriteLine("Required parameters missing " );
        System.Console.WriteLine("Usage: exmqreq1 'request queue name' 'response queue name' 'queue manager name'" );
        System.Console.WriteLine("   eg. exmqreq1 LEXQ1 LEXQ2 EXQMAN " );
        return;
      }
      else 
      {
        requestQueueName = args[0];
        responseQueueName = args[1];
      }

      ///
      /// create an MQRCText instance
      /// 
       
      mqrcText = new MQRCText();
       
      ///
      /// Try to create an MQQueueManager instance 
      /// 
      try 
      {
        if (args.Length > 2)
        {
          // queue name, queue manager name provided
          mqQMgr = new MQQueueManager( args[2] );
        }
        else 
        {
          if (args[0] == "?")
          {
           System.Console.WriteLine("Usage: exmqreq1 'request queue name' 'response queue name' 'queue manager name'" );
           System.Console.WriteLine("   eg. exmqreq1 LEXQ1 LEXQ2 EXQMAN " );
           return;
          }
          else
          {
            System.Console.WriteLine("Required parameter missing - queue manager name" );
            return;
          }
        }
      }
      catch (MQException mqe) 
      {
        // stop if failed
        String mText = mqrcText.getMQRCText(mqe.Reason);     // get the message text
        System.Console.WriteLine( "create of MQQueueManager ended with " + mqe.Message + " Text: " + mText );
        return;
      }

      Console.WriteLine("Example exmqreq1 started");

      ///
      /// Try to open the request queue
      ///
      try 
      {
        requestQueue = mqQMgr.AccessQueue(requestQueueName,
                                          MQC.MQOO_OUTPUT                   // open queue for output
                                        + MQC.MQOO_FAIL_IF_QUIESCING );     // but not if MQM stopping
      }
      catch (MQException mqe) 
      {
        // stop if failed
        String mText = mqrcText.getMQRCText(mqe.Reason);      // get the message text
        System.Console.WriteLine( "Open request queue ended with " + mqe.Message + " Text: " + mText );
        return;
      }

      ///
      /// Try to open the response queue
      ///
      try 
      {
        responseQueue = mqQMgr.AccessQueue(responseQueueName,
                                           MQC.MQOO_INPUT_AS_Q_DEF           // open queue for input
                                         + MQC.MQOO_FAIL_IF_QUIESCING );     // but not if MQM stopping
      }
      catch (MQException mqe) 
      {
        // stop if failed
        String mText = mqrcText.getMQRCText(mqe.Reason);      // get the message text
        System.Console.WriteLine( "Open response queue ended with " + mqe.Message + " Text: " + mText );
        return;
      }

      ///
      /// Read lines from the console and put them to the request message queue
      /// Wait for reponse                                
      /// Read lines from the response message queue and write them to the console
      /// 
      try
      {

        Console.WriteLine("Enter request text: ");
        string line = Console.ReadLine();

        MQMessage requestMessage = new MQMessage();
        requestMessage.WriteString(line);
        requestMessage.Format = MQC.MQFMT_STRING;
        requestMessage.MessageType = MQC.MQMT_REQUEST;
        requestMessage.Report = MQC.MQRO_COPY_MSG_ID_TO_CORREL_ID;
        requestMessage.ReplyToQueueName = responseQueueName;
        requestQueue.Put(requestMessage);                             // put request message
        requestQueue.Close();

        Console.WriteLine("Request sent, waiting for response ...");

        MQMessage responseMessage = new MQMessage();
        responseMessage.CorrelationId = requestMessage.MessageId;     // response correlation id has to match
                                                                      // the request message id
        MQGetMessageOptions gmo = new MQGetMessageOptions();
        gmo.Options = MQC.MQGMO_WAIT;                                 // tell get to wait
        gmo.WaitInterval = 60 * 1000;                                 // tell get to wait up to 60 seconds
        gmo.MatchOptions = MQC.MQMO_MATCH_CORREL_ID;

        try
        {
          responseQueue.Get(responseMessage, gmo);                    // get response message
          line = responseMessage.ReadString(responseMessage.MessageLength);
          Console.WriteLine("Response: {0}", line);
        }
        catch (MQException mqe)
        {
          String mText = mqrcText.getMQRCText(mqe.Reason);
          if ( mText.Equals("MQRC_NO_MSG_AVAILABLE"))
          {
            // special report for normal end
            System.Console.WriteLine( "no more messages on queue LEXQ2" );
          }
          else
          {
            System.Console.WriteLine( "Read response queue ended with " + mqe.Message + " Text: " + mText );
          }
        }
        responseQueue.Close();
        mqQMgr.Disconnect();
      }
      catch(MQException mqe)
      {
        String mText = mqrcText.getMQRCText(mqe.Reason);
        System.Console.WriteLine( "Write request queue ended with " + mqe.Message + " Text: " + mText );
      }


      System.Console.WriteLine( "Example exmqreq1 end" );
      return;
    }
  }
}

