// This application demonstrates sending a request message to a named queue
// and awaiting a response.  The response is written to a dynamic queue
// specified at the time of the open request queue. The format of the dynamic
// queue is "name entered as parameter 2"+userid+time
// For the response to arrive, a partner application must receive the request
// and generate the response to the dynamic queue.  Program EXMQGET3 is used 
// for this purpose.
//
// If no response arrives within 60 seconds, the request is assumed to have
// timed out.
//
//      program name:  exmqreq2 - example of MQPUT to write a message to a queue
//                                and receive a reply in a second queue         
//                
//                exmqreq2 has 2 parameters:
//                - the name of a request queue (required)
//                - the name of a queue manager (required)
//
//    Usage: exmqreq2 RequestQueueName QmgrName
//    
//    example command line:   exmqreq2 LEXQ1 EXQMAN

using System;
using IBM.WMQ;

namespace exmqreq2
{
  /// <summary>
  /// Summary description for exmqreq2.
  /// </summary>
  class exmqreq2
  {
    /// <summary>
    /// The main entry point for the application.
    /// </summary>
    [STAThread]
    static void Main(String[] args)
    {
      MQQueueManager      mqQMgr;            // MQQueueManager instance
      MQQueue             requestQueue;      // MQQueue instance
      MQQueue             responseQueue;     // MQQueue instance
      MQRCText            mqrcText;          // MQRCText instance
      String              mqQMgrName = "";
      String              requestQueueName;  // Name of queue to use for requests
      String              responseQueueName; // Name of queue to use for requests
      String              queueNameModel = "SYSTEM.DEFAULT.MODEL.QUEUE"; // Name of model queue to use 
       
      String userId = Environment.UserName;              // get user ID
      System.DateTime DT = System.DateTime.Now;          // get time and string it together as hhmmssmmm
      String hour = DT.Hour.ToString();                  // these fields will be used to build a unique
      String minute = DT.Minute.ToString();              // dynamic queue name
      String second = DT.Second.ToString();
      String millisecond = DT.Millisecond.ToString();
      String TimeOfDay = hour+minute+second+millisecond;

      if (args.Length == 0) 
      {
        System.Console.WriteLine("Required parameters missing " );
        System.Console.WriteLine("Usage: exmqreq2 'request queue name' 'response queue name' 'queue manager name'" );
        System.Console.WriteLine("   eg. exmqreq2 LEXQ1 LEXQ2 EXQMAN " );
        return;
      }
      else 
      {
        requestQueueName = args[0];
        responseQueueName = "dynam"+userId+TimeOfDay;    // build unique dynamic queue name
        Console.WriteLine("dynamic responseQueueName: {0}", responseQueueName);
      }

      ///
      /// create an MQRCText instance
      /// 
       
      mqrcText = new MQRCText();                        // create instance of MQ error message text class
       
      ///
      /// Try to create an MQQueueManager instance 
      /// 
      try 
      {
        if (args.Length > 1)
        {
          // queue name, queue manager name provided
          mqQMgr = new MQQueueManager( args[1] );
          mqQMgrName = args[1];
        }
        else 
        {
          if (args[0] == "?")
          {
           System.Console.WriteLine("Usage: exmqreq2 'request queue name' 'queue manager name'" );
           System.Console.WriteLine("   eg. exmqreq2 LEXQ1 EXQMAN " );
           return;
          }
          else
          {
            System.Console.WriteLine("Required parameter missing - queue manager name" );
            return;
          }
        }
      }
      catch (MQException mqe) 
      {
        // stop if failed
        String mText = mqrcText.getMQRCText(mqe.Reason);     // get the message text
        System.Console.WriteLine( "create of MQQueueManager ended with " + mqe.Message + " Text: " + mText );
        return;
      }

      Console.WriteLine("Example exmqreq2 started");

      ///
      /// Try to open the request queue
      ///
      try 
      {
        requestQueue = mqQMgr.AccessQueue(requestQueueName,
                                          MQC.MQOO_OUTPUT                   // open queue for output
                                        + MQC.MQOO_FAIL_IF_QUIESCING );     // but not if MQM stopping
      }
      catch (MQException mqe) 
      {
        // stop if failed
        String mText = mqrcText.getMQRCText(mqe.Reason);      // get the message text
        System.Console.WriteLine( "Open request queue ended with " + mqe.Message + " Text: " + mText );
        return;
      }

      ///
      /// Try to open the response queue
      ///
      try 
      {
        responseQueue = mqQMgr.AccessQueue(queueNameModel,
                                           MQC.MQOO_INPUT_EXCLUSIVE          // open dynamic queue for reply
                                         + MQC.MQOO_FAIL_IF_QUIESCING,       // but not if MQM stopping
                                           mqQMgrName, responseQueueName, null );   
      }
      catch (MQException mqe) 
      {
        // stop if failed
        String mText = mqrcText.getMQRCText(mqe.Reason);      // get the message text
        System.Console.WriteLine( "Open response queue ended with " + mqe.Message + " Text: " + mText );
        return;
      }

      ///
      /// Read line from the console and put it to the request message queue
      /// Wait for reponse                                
      /// Read line from the response message queue and write it to the console
      /// 
      try
      {

        Console.WriteLine("Enter request text: ");
        string line = Console.ReadLine();

        MQMessage requestMessage = new MQMessage();
        requestMessage.WriteString(line);
        requestMessage.Format = MQC.MQFMT_STRING;
        requestMessage.MessageType = MQC.MQMT_REQUEST;
        requestMessage.Report = MQC.MQRO_COPY_MSG_ID_TO_CORREL_ID;
        requestMessage.ReplyToQueueName = responseQueueName;
        requestQueue.Put(requestMessage);                             // put request message
        requestQueue.Close();

        Console.WriteLine("Request sent, waiting for response ...");

        MQMessage responseMessage = new MQMessage();
        responseMessage.CorrelationId = requestMessage.MessageId;     // response correlation id has to match
                                                                      // the request message id
        MQGetMessageOptions gmo = new MQGetMessageOptions();
        gmo.Options = MQC.MQGMO_WAIT;                                 // tell get to wait 
        gmo.WaitInterval = 60 * 1000;
        gmo.MatchOptions = MQC.MQMO_MATCH_CORREL_ID;                  // tell get to wait up to 60 seconds

        try
        {
          responseQueue.Get(responseMessage, gmo);                    // get response message
          line = responseMessage.ReadString(responseMessage.MessageLength);
          Console.WriteLine("Response: {0}", line);
        }
        catch (MQException mqe)
        {
          String mText = mqrcText.getMQRCText(mqe.Reason);
          if ( mText.Equals("MQRC_NO_MSG_AVAILABLE"))
          {
            // special report for normal end
            System.Console.WriteLine( "no more messages on dynamic reply queue" );
          }
          else
          {
            System.Console.WriteLine( "Read response queue ended with " + mqe.Message + " Text: " + mText );
          }
        }
        responseQueue.Close();
        mqQMgr.Disconnect();
      }
      catch(MQException mqe)
      {
        String mText = mqrcText.getMQRCText(mqe.Reason);
        System.Console.WriteLine( "Write request queue ended with " + mqe.Message + " Text: " + mText );
      }


      System.Console.WriteLine( "Example exmqreq2 end" );
      return;
    }
  }
}

